@@ -97,27 +97,32 @@
 ;;; Here we define some constants (markers, quantization and huffman tables etc.)
 
 (eval-when (:compile-toplevel :load-toplevel)
+  
+(defmacro defconstant* (name value &optional doc)
+  "Ensure VALUE is evaluated only once, required for ANSI and SBCL."
+  `(defconstant ,name (if (boundp ',name) (symbol-value ',name) ,value)
+     ,@(when doc (list doc))))
 
 ;;; Source huffman tables for the encoder
-(defconstant *luminance-dc-bits*
+(defconstant* *luminance-dc-bits*
   #(#x00 #x01 #x05 #x01 #x01 #x01 #x01 #x01
      #x01 #x00 #x00 #x00 #x00 #x00 #x00 #x00))
 
-(defconstant *luminance-dc-values*
+(defconstant* *luminance-dc-values*
   #(#x00 #x01 #x02 #x03 #x04 #x05 #x06 #x07 #x08 #x09 #x0a #x0b))
 
-(defconstant *chrominance-dc-bits*
+(defconstant* *chrominance-dc-bits*
   #(#x00 #x03 #x01 #x01 #x01 #x01 #x01 #x01
      #x01 #x01 #x01 #x00 #x00 #x00 #x00 #x00))
 
-(defconstant *chrominance-dc-values*
+(defconstant* *chrominance-dc-values*
   #(#x00 #x01 #x02 #x03 #x04 #x05 #x06 #x07 #x08 #x09 #x0a #x0b))
 
-(defconstant *luminance-ac-bits*
+(defconstant* *luminance-ac-bits*
   #(#x00 #x02 #x01 #x03 #x03 #x02 #x04 #x03
      #x05 #x05 #x04 #x04 #x00 #x00 #x01 #x7d))
 
-(defconstant *luminance-ac-values*
+(defconstant* *luminance-ac-values*
   #(#x01 #x02 #x03 #x00 #x04 #x11 #x05 #x12
      #x21 #x31 #x41 #x06 #x13 #x51 #x61 #x07
      #x22 #x71 #x14 #x32 #x81 #x91 #xa1 #x08
@@ -140,11 +145,11 @@
      #xf1 #xf2 #xf3 #xf4 #xf5 #xf6 #xf7 #xf8
      #xf9 #xfa))
 
-(defconstant *chrominance-ac-bits*
+(defconstant* *chrominance-ac-bits*
   #(#x00 #x02 #x01 #x02 #x04 #x04 #x03 #x04
      #x07 #x05 #x04 #x04 #x00 #x01 #x02 #x77))
 
-(defconstant *chrominance-ac-values*
+(defconstant* *chrominance-ac-values*
   #(#x00 #x01 #x02 #x03 #x11 #x04 #x05 #x21
      #x31 #x06 #x12 #x41 #x51 #x07 #x61 #x71
      #x13 #x22 #x32 #x81 #x08 #x14 #x42 #x91
@@ -168,7 +173,7 @@
      #xf9 #xfa))
 
 ;;;Zigzag encoding matrix
-(defconstant *zigzag-index*
+(defconstant* *zigzag-index*
   #(#(0  1  5  6 14 15 27 28)
     #(2  4  7 13 16 26 29 42)
     #(3  8 12 17 25 30 41 43)
@@ -181,7 +186,7 @@
 ;;; Temporary buffer for zigzag encoding and decoding
 (defvar *zz-result* (make-array 64 :element-type 'unsigned-byte))
 
-(defconstant *zzbuf*
+(defconstant* *zzbuf*
   #(#(0  0  0  0  0  0  0  0)
     #(0  0  0  0  0  0  0  0)
     #(0  0  0  0  0  0  0  0)
@@ -290,10 +295,10 @@
 (finish-output)
 )
 
-(defconstant *q-tables* (vector *q-luminance* *q-chrominance*))
+(defconstant* *q-tables* (vector *q-luminance* *q-chrominance*))
 
 ;;; This table is used to map coefficients into SSSS value
-(defconstant *csize* (make-array 2047 
+(defconstant* *csize* (make-array 2047 
 				 :initial-contents
 				 (loop for i fixnum from 0 to 2046
 				       collecting (integer-length (abs (minus i 1023))))))
